# model2/__init__.py



from .dataloader import DataProcessor  # DataProcessor 클래스는 데이터를 처리하는 기능을 포함합니다.
from .model import CNNGRUClassificationModel
from .predictor import Predictor
# from .predict import predict